@@ -45,7 +45,7 @@
 #include "errorgui.h"
 #include "gdm-net.h"
 
-static const gchar RCSid[]="$Id: gdm.c,v 1.67 2001/08/26 01:56:37 jirka Exp $";
+static const gchar RCSid[]="$Id: gdm.c,v 1.69 2001/09/01 09:06:30 jirka Exp $";
 
 
 /* Local functions */
@@ -135,6 +135,7 @@ gboolean  GdmIndirect = FALSE;
 gint  GdmMaxIndirect = 0;
 gint  GdmMaxIndirectWait = 0;
 gint  GdmPingInterval = 0;
+gchar *GdmWilling = NULL;
 gboolean  GdmDebug = FALSE;
 gboolean  GdmVerboseAuth = FALSE;
 gboolean  GdmAllowRoot = FALSE;
@@ -156,6 +157,19 @@ char *stored_path = NULL;
 
 static gboolean gdm_restart_mode = FALSE;
 
+static gboolean
+display_exists (int num)
+{
+	GSList *li;
+
+	for (li = displays; li != NULL; li = li->next) {
+		GdmDisplay *disp = li->data;
+		if (disp->dispnum == num)
+			return TRUE;
+	}
+	return FALSE;
+}
+
 
 /**
  * gdm_config_parse:
@@ -242,10 +256,13 @@ gdm_config_parse (void)
     GdmMaxIndirect = gnome_config_get_int (GDM_KEY_MAXINDIR);
     GdmMaxIndirectWait = gnome_config_get_int (GDM_KEY_MAXINDWAIT);    
     GdmPingInterval = gnome_config_get_int (GDM_KEY_PINGINTERVAL);    
+    GdmWilling = gnome_config_get_string (GDM_KEY_WILLING);    
 
     GdmStandardXServer = gnome_config_get_string (GDM_KEY_STANDARD_XSERVER);    
-    if (ve_string_empty (GdmStandardXServer))
+    if (ve_string_empty (GdmStandardXServer)) {
+	    g_free (GdmStandardXServer);
 	    GdmStandardXServer = g_strdup ("/usr/bin/X11/X");
+    }
     GdmFlexibleXServers = gnome_config_get_int (GDM_KEY_FLEXIBLE_XSERVERS);    
     GdmXnest = gnome_config_get_string (GDM_KEY_XNEST);    
     if (ve_string_empty (GdmXnest))
@@ -373,7 +390,18 @@ gdm_config_parse (void)
     while (iter) {
 	    if (isdigit (*k)) {
 		    int disp_num = atoi (k);
-		    GdmDisplay *disp = gdm_server_alloc (disp_num, v);
+		    GdmDisplay *disp;
+
+		    while (display_exists (disp_num)) {
+			    disp_num++;
+		    }
+
+		    if (disp_num != atoi (k)) {
+			    gdm_error (_("%s: Display number %d in use!  I will use %d"),
+				       "gdm_config_parse", atoi (k), disp_num);
+		    }
+
+		    disp = gdm_server_alloc (disp_num, v);
 		    if (disp == NULL) {
 			    g_free (k);
 			    g_free (v);
@@ -464,7 +492,7 @@ gdm_config_parse (void)
     /* Check that the greeter can be executed */
     bin = ve_first_word (GdmGreeter);
 
-    if ( ! ve_string_empty (bin) &&
+    if (ve_string_empty (bin) ||
 	access (bin, X_OK) != 0) {
 	    gdm_error (_("%s: Greeter not found or can't be executed by the gdm user"), "gdm_config_parse");
     }
@@ -476,8 +504,8 @@ gdm_config_parse (void)
     bin = ve_first_word (GdmChooser);
 
     if (GdmIndirect &&
-	! ve_string_empty (bin) &&
-	access (bin, X_OK) != 0) {
+	(ve_string_empty (bin) ||
+	 access (bin, X_OK) != 0)) {
 	    gdm_error (_("%s: Chooser not found or it can't be executed by the gdm user"), "gdm_config_parse");
     }
     
@@ -1703,6 +1731,7 @@ gdm_handle_message (GdmConnection *conn, const char *msg, gpointer data)
 								 NULL, NULL);
 				if ( ! gdm_connection_write (conn, msg))
 					gdm_display_unmanage (d);
+				g_free (msg);
 			}
 			
 			gdm_debug ("Got FLEXI_OK");
@@ -1790,8 +1819,8 @@ handle_flexi_server (GdmConnection *conn, int type, const char *server,
 		return;
 	}
 
-	bin = ve_first_word (ve_sure_string (server));
-	if (server == NULL ||
+	bin = ve_first_word (server);
+	if (ve_string_empty (server) ||
 	    access (bin, X_OK) != 0) {
 		g_free (bin);
 		gdm_connection_write (conn,
@@ -1923,7 +1952,6 @@ gdm_handle_user_message (GdmConnection *conn, const char *msg, gpointer data)
 		}
 
 		svr = gdm_find_x_server (name);
-		g_free (name);
 		if (svr == NULL) {
 			/* Don't print the name to syslog as it might be
 			 * long and dangerous */
@@ -1938,6 +1966,7 @@ gdm_handle_user_message (GdmConnection *conn, const char *msg, gpointer data)
 		} else {
 			command = svr->command;
 		}
+		g_free (name);
 
 		handle_flexi_server (conn, TYPE_FLEXI, command, NULL, NULL);
 	} else if (strncmp (msg, GDM_SUP_FLEXI_XNEST " ",
