@@ -23,6 +23,7 @@
 #include <X11/Xlib.h>
 #include <X11/Xmd.h>
 #include <X11/Xauth.h>
+#include <netinet/in.h>
 
 #define STX 0x2			/* Start of txt */
 #define BEL 0x7			/* Bell, used to interrupt login for
@@ -176,6 +177,7 @@
 #define GDM_KEY_MAXINDIR "xdmcp/MaxPendingIndirect=4"
 #define GDM_KEY_MAXINDWAIT "xdmcp/MaxWaitIndirect=30"
 #define GDM_KEY_PINGINTERVAL "xdmcp/PingInterval=5"
+#define GDM_KEY_WILLING "xdmcp/Willing=" EXPANDED_SYSCONFDIR "/gdm/Xwilling"
 
 #define GDM_KEY_GTKRC "gui/GtkRC=" EXPANDED_DATADIR "/themes/Default/gtk/gtkrc"
 #define GDM_KEY_ICONWIDTH "gui/MaxIconWidth=128"
@@ -253,6 +255,7 @@ struct _GdmDisplay {
     gchar *bcookie;
     gchar *name;
     gchar *hostname;
+    struct in_addr addr;
     guint8 dispstat;
     guint16 dispnum;
     guint8 servstat;
@@ -327,7 +330,7 @@ struct _GdmForwardQuery {
 
 /* some extra xdmcp opcodes that xdm will happily ignore since they'll be
  * the wrong xdmcp version anyway */
-#define GDM_XDMCP_PROTOCOL_VERSION 1000
+#define GDM_XDMCP_PROTOCOL_VERSION 1001
 enum {
 	GDM_XDMCP_FIRST_OPCODE = 1000, /*just a marker, not an opcode */
 
@@ -340,7 +343,7 @@ enum {
 		 * dropped on the floor, however, it is not completely
 		 * necessary for it to get through.
 		 *
-		 * Argument is ARRAY8 with the hostname of the manager */
+		 * Argument is ARRAY8 with the address of the originating host */
 	GDM_XDMCP_LAST_OPCODE /*just a marker, not an opcode */
 };
 
