@@ -27,28 +27,15 @@ void gdm_info   (const gchar *format, ...) G_GNUC_PRINTF (1, 2);
 void gdm_error  (const gchar *format, ...) G_GNUC_PRINTF (1, 2);
 void gdm_debug  (const gchar *format, ...) G_GNUC_PRINTF (1, 2);
 
-#ifdef HAVE_SETENV
-#define gdm_setenv(var,value) setenv(var,value,-1)
-#else
-gint gdm_setenv (const gchar *var, const gchar *value);
-#endif
-
-#ifdef HAVE_UNSETENV
-#define gdm_unsetenv(var) unsetenv(var)
-#else
-gint gdm_unsetenv (const gchar *var);
-#endif
-
-void gdm_clearenv (void);
-
 /* clear environment, but keep the i18n ones (LANG, LC_ALL, etc...),
  * note that this leak memory so only use before exec */
 void gdm_clearenv_no_lang (void);
 
-int gdm_strcasecmp_no_locale (const char *s1, const char *s2);
+int gdm_get_free_display (int start, int server_uid);
 
-char ** gdm_split (const char *s);
-char * gdm_first_word (const char *s);
+gboolean gdm_text_message_dialog (const char *msg);
+gboolean gdm_text_yesno_dialog (const char *msg, gboolean *ret);
+int	gdm_exec_wait (char * const *argv, gboolean no_display);
 
 #endif /* GDM_MISC_H */
 
