@@ -32,7 +32,7 @@
 #include "slave.h"
 #include "verify.h"
 
-static const gchar RCSid[]="$Id: verify-crypt.c,v 1.14 2001/08/10 19:14:18 jirka Exp $";
+static const gchar RCSid[]="$Id: verify-crypt.c,v 1.16 2001/08/22 23:11:54 jirka Exp $";
 
 
 /* Configuration option variables */
@@ -166,7 +166,9 @@ gdm_verify_user (const char *username,
 
     /* check for the standard method of disallowing users */
     if (pwent->pw_shell != NULL &&
-	strcmp (pwent->pw_shell, "/bin/false") == 0) {
+	(strcmp (pwent->pw_shell, "/sbin/nologin") == 0 ||
+	 strcmp (pwent->pw_shell, "/bin/true") == 0 ||
+	 strcmp (pwent->pw_shell, "/bin/false") == 0)) {
 	    gdm_error (_("User %s not allowed to log in"), login);
 	    if (GdmVerboseAuth) {
 		    gdm_slave_greeter_ctl_no_ret (GDM_ERRBOX,
@@ -231,9 +233,10 @@ gdm_verify_check (void)
 }
 
 /* used in pam */
-void
-gdm_verify_env_setup (void)
+gboolean
+gdm_verify_open_session (void)
 {
+	return TRUE;
 }
 
 /* EOF */
