@@ -59,6 +59,7 @@
 
 #ifdef HAVE_LIBXDMCP
 #include <stdio.h>
+#include <time.h>
 #include <stdlib.h>
 #include <syslog.h>
 #include <sys/stat.h>
@@ -95,7 +96,7 @@
 #include "choose.h"
 #include "xdmcp.h"
 
-static const gchar RCSid[]="$Id: xdmcp.c,v 1.35 2001/08/22 23:11:54 jirka Exp $";
+static const gchar RCSid[]="$Id: xdmcp.c,v 1.40 2001/09/01 12:31:58 jirka Exp $";
 gint pending = 0;
 
 #ifdef HAVE_LIBXDMCP
@@ -126,6 +127,7 @@ extern gint GdmMaxIndirectWait;	/* Max wait between INDIRECT_QUERY and MANAGE */
 extern gint GdmDispPerHost;	/* Max number of displays per remote host */
 extern gchar *GdmTimedLogin;
 extern gboolean GdmAllowRemoteAutoLogin;
+extern gchar *GdmWilling;	/* The willing script */
 
 extern gboolean GdmXdmcp;	/* xdmcp enabled */
 
@@ -148,16 +150,18 @@ static void gdm_xdmcp_handle_keepalive (struct sockaddr_in *clnt_sa, gint len);
 static void gdm_xdmcp_send_willing (struct sockaddr_in *clnt_sa);
 static void gdm_xdmcp_send_unwilling (struct sockaddr_in *clnt_sa, gint type);
 static void gdm_xdmcp_send_accept (const char *hostname, struct sockaddr_in *clnt_sa, gint displaynum);
-static void gdm_xdmcp_send_decline (struct sockaddr_in *clnt_sa);
+static void gdm_xdmcp_send_decline (struct sockaddr_in *clnt_sa, const char *reason);
 static void gdm_xdmcp_send_refuse (struct sockaddr_in *clnt_sa, CARD32 sessid);
 static void gdm_xdmcp_send_failed (struct sockaddr_in *clnt_sa, CARD32 sessid);
 static void gdm_xdmcp_send_alive (struct sockaddr_in *clnt_sa, CARD32 sessid);
-static void gdm_xdmcp_send_managed_forward (struct sockaddr_in *clnt_sa);
+static void gdm_xdmcp_send_managed_forward (struct sockaddr_in *clnt_sa,
+					    struct sockaddr_in *origin);
 static gboolean gdm_xdmcp_host_allow (struct sockaddr_in *cnlt_sa);
-static GdmDisplay *gdm_xdmcp_display_alloc (const char *hostname, gint);
+static GdmDisplay *gdm_xdmcp_display_alloc (struct in_addr *addr, const char *hostname, gint);
 static GdmDisplay *gdm_xdmcp_display_lookup (CARD32 sessid);
 static void gdm_xdmcp_display_dispose_check (const gchar *name);
 static void gdm_xdmcp_displays_check (void);
+static int gdm_xdmcp_displays_from_host (struct in_addr *addr);
 
 static GdmForwardQuery * gdm_forward_query_alloc (struct sockaddr_in *mgr_sa,
 						  struct sockaddr_in *dsp_sa);
@@ -231,6 +235,7 @@ peek_local_address_list (void)
 	if (ioctl (xdmcpfd, SIOCGIFCONF, &ifc) < 0) {
 		gdm_error (_("%s: Cannot get local addresses!"),
 			   "peek_local_address_list");
+		g_free (buf);
 		return NULL;
 	}
 
@@ -287,44 +292,22 @@ peek_local_address_list (void)
 	return the_list;
 }
 
-
-static gboolean
-is_loopback_addr (struct in_addr *ia)
+static int
+gdm_xdmcp_displays_from_host (struct in_addr *addr)
 {
-	const char lo[] = {127,0,0,1};
+	GSList *li;
+	int count = 0;
 
-	if (ia->s_addr == INADDR_LOOPBACK ||
-	    memcmp (&ia->s_addr, lo, 4) == 0) {
-		return TRUE;
-	} else {
-		return FALSE;
+	for (li = displays; li != NULL; li = li->next) {
+		GdmDisplay *disp = li->data;
+		if (disp->type == TYPE_XDMCP &&
+		    memcmp (&disp->addr, addr, sizeof (struct in_addr)) == 0)
+			count ++;
 	}
+
+	return count;
 }
 
-static gboolean
-is_local_addr (struct in_addr *ia)
-{
-	const char lo[] = {127,0,0,1};
-
-	if (ia->s_addr == INADDR_LOOPBACK ||
-	    memcmp (&ia->s_addr, lo, 4) == 0) {
-		return TRUE;
-	} else {
-		const GList *list = peek_local_address_list ();
-
-		while (list != NULL) {
-			struct in_addr *addr = list->data;
-
-			if (memcmp (&ia->s_addr, &addr->s_addr, 4) == 0) {
-				return TRUE;
-			}
-
-			list = list->next;
-		}
-
-		return FALSE;
-	}
-}
 
 gboolean
 gdm_xdmcp_init (void)
@@ -530,12 +513,12 @@ gdm_xdmcp_handle_query (struct sockaddr_in *clnt_sa, gint len, gint type)
 		if (id != NULL &&
 		    id->chosen_host != NULL) {
 			/* if user chose us, then just send willing */
-			if (is_local_addr (id->chosen_host)) {
+			if (gdm_xdmcp_is_local_addr (id->chosen_host)) {
 				/* get rid of indirect, so that we don't get
 				 * the chooser */
 				gdm_choose_indirect_dispose (id);
 				gdm_xdmcp_send_willing (clnt_sa);
-			} else if (is_loopback_addr (&(clnt_sa->sin_addr))) {
+			} else if (gdm_xdmcp_is_loopback_addr (&(clnt_sa->sin_addr))) {
 				/* woohoo! fun, I have no clue how to get
 				 * the correct ip, SO I just send forward
 				 * queries with all the different IPs */
@@ -544,7 +527,7 @@ gdm_xdmcp_handle_query (struct sockaddr_in *clnt_sa, gint len, gint type)
 				while (list != NULL) {
 					struct in_addr *addr = list->data;
 					
-					if ( ! is_loopback_addr (addr)) {
+					if ( ! gdm_xdmcp_is_loopback_addr (addr)) {
 						/* forward query to
 						 * chosen host */
 						gdm_xdmcp_send_forward_query
@@ -831,11 +814,45 @@ gdm_xdmcp_send_willing (struct sockaddr_in *clnt_sa)
 {
     ARRAY8 status;
     XdmcpHeader header;
+    static char *last_status = NULL;
+    static time_t last_willing = 0;
+    char *bin;
+    FILE *fd;
     
     gdm_debug ("gdm_xdmcp_send_willing: Sending WILLING to %s", inet_ntoa (clnt_sa->sin_addr));
-    
-    status.data = sysid;
-    status.length = strlen (sysid);
+
+    if (last_willing == 0 ||
+	time (NULL) - 3 > last_willing) {
+	    char statusBuf[256] = "";
+	    bin = ve_first_word (GdmWilling);
+	    if ( ! ve_string_empty (bin) &&
+		 access (bin, X_OK) == 0 &&
+		 (fd = popen (GdmWilling, "r")) != NULL) {
+		    if (fgets (statusBuf, sizeof (statusBuf), fd) != NULL &&
+			! ve_string_empty (g_strstrip (statusBuf))) {
+			    g_free (last_status);
+			    last_status = g_strdup (statusBuf);
+		    } else {
+			    g_free (last_status);
+			    last_status = g_strdup (sysid);
+		    }
+		    fclose (fd);
+	    } else {
+		    g_free (last_status);
+		    last_status = g_strdup (sysid);
+	    }
+	    last_willing = time (NULL);
+    }
+
+    if ( ! gdm_xdmcp_is_local_addr (&(clnt_sa->sin_addr)) &&
+	 gdm_xdmcp_displays_from_host (&(clnt_sa->sin_addr)) >= GdmDispPerHost) {
+	    /* Don't translate, this goes over the wire to servers where we
+	     * don't know the charset or language, so it must be ascii */
+	    status.data = g_strdup_printf ("%s (Server is busy)", last_status);
+    } else {
+	    status.data = g_strdup (last_status);
+    }
+    status.length = strlen (status.data);
     
     header.opcode = (CARD16) WILLING;
     header.length = 6 + serv_authlist.authentication.length;
@@ -848,6 +865,8 @@ gdm_xdmcp_send_willing (struct sockaddr_in *clnt_sa)
     XdmcpWriteARRAY8 (&buf, &status);
     XdmcpFlush (xdmcpfd, &buf, (XdmcpNetaddr)clnt_sa,
 		(int)sizeof (struct sockaddr_in));
+
+    g_free (status.data);
 }
 
 static void
@@ -860,7 +879,9 @@ gdm_xdmcp_send_unwilling (struct sockaddr_in *clnt_sa, gint type)
     
     gdm_error (_("Denied XDMCP query from host %s"), inet_ntoa (clnt_sa->sin_addr));
     
-    status.data = _("Display not authorized to connect");
+    /* Don't translate, this goes over the wire to servers where we
+     * don't know the charset or language, so it must be ascii */
+    status.data = "Display not authorized to connect";
     status.length = strlen (status.data);
     
     header.opcode = (CARD16) UNWILLING;
@@ -875,28 +896,25 @@ gdm_xdmcp_send_unwilling (struct sockaddr_in *clnt_sa, gint type)
 }
 
 static void
-gdm_xdmcp_send_managed_forward (struct sockaddr_in *clnt_sa)
+gdm_xdmcp_send_managed_forward (struct sockaddr_in *clnt_sa,
+				struct sockaddr_in *origin)
 {
-	ARRAY8 hostname;
+	ARRAY8 address;
 	XdmcpHeader header;
+	struct in_addr addr;
 
 	gdm_debug ("gdm_xdmcp_send_managed_forward: Sending MANAGED_FORWARD to %s", inet_ntoa (clnt_sa->sin_addr));
 
-	hostname.data = g_new0 (char, 1024);
-	if (gethostname (hostname.data, 1023) != 0) {
-		g_free (hostname.data);
-		/* eek ! */
-		return;
-	}
-
-	hostname.length = strlen (hostname.data);
+	address.length = sizeof (struct in_addr);
+	address.data = (void *)&addr;
+	memcpy (address.data, &(origin->sin_addr), sizeof (struct in_addr));
 
 	header.opcode = (CARD16) GDM_XDMCP_MANAGED_FORWARD;
-	header.length = 4 + hostname.length;
+	header.length = 4 + address.length;
 	header.version = GDM_XDMCP_PROTOCOL_VERSION;
 	XdmcpWriteHeader (&buf, &header);
 
-	XdmcpWriteARRAY8 (&buf, &hostname);
+	XdmcpWriteARRAY8 (&buf, &address);
 	XdmcpFlush (xdmcpfd, &buf, (XdmcpNetaddr)clnt_sa,
 		    (int)sizeof (struct sockaddr_in));
 }
@@ -908,7 +926,7 @@ get_host_from_addr (struct sockaddr_in *clnt_sa)
 	struct hostent *he;
 
 	/* Find client hostname */
-	he = gethostbyaddr ((gchar *) &clnt_sa->sin_addr,
+	he = gethostbyaddr ((gchar *) &(clnt_sa->sin_addr),
 			    sizeof (struct in_addr),
 			    AF_INET);
 
@@ -1034,9 +1052,12 @@ gdm_xdmcp_handle_request (struct sockaddr_in *clnt_sa, gint len)
     gdm_debug ("gdm_xdmcp_handle_request: pending=%d, MaxPending=%d, sessions=%d, MaxSessions=%d",
 	       pending, GdmMaxPending, sessions, GdmMaxSessions);
 
+
     /* Check if ok to manage display */
     if (mitauth &&
-	sessions < GdmMaxSessions) {
+	sessions < GdmMaxSessions &&
+	(gdm_xdmcp_is_local_addr (&(clnt_sa->sin_addr)) ||
+	 gdm_xdmcp_displays_from_host (&(clnt_sa->sin_addr)) < GdmDispPerHost)) {
 	    char *disp;
 	    char *hostname = get_host_from_addr (clnt_sa);
 	    disp = g_strdup_printf ("%s:%d", hostname, clnt_dspnum);
@@ -1047,14 +1068,23 @@ gdm_xdmcp_handle_request (struct sockaddr_in *clnt_sa, gint len)
 
 	    if (pending >= GdmMaxPending) {
 		    gdm_debug ("gdm_xdmcp_handle_request: maximum pending");
-		    gdm_xdmcp_send_decline (clnt_sa);	
+		    /* Don't translate, this goes over the wire to servers where we
+		    * don't know the charset or language, so it must be ascii */
+		    gdm_xdmcp_send_decline (clnt_sa, "Maximum pending servers");	
 	    } else {
 		    gdm_xdmcp_send_accept (hostname, clnt_sa, clnt_dspnum);
 	    }
 
 	    g_free (hostname);
     } else {
-	    gdm_xdmcp_send_decline (clnt_sa);	
+	    /* Don't translate, this goes over the wire to servers where we
+	    * don't know the charset or language, so it must be ascii */
+	    if ( ! mitauth)
+		    gdm_xdmcp_send_decline (clnt_sa, "Only MIT-MAGIC-COOKIE-1 supported");	
+	    else if (sessions < GdmMaxSessions)
+		    gdm_xdmcp_send_decline (clnt_sa, "Maximum number of open sessions reached");	
+	    else 
+		    gdm_xdmcp_send_decline (clnt_sa, "Maximum number of open sessions from your host reached");	
     }
 
     XdmcpDisposeARRAY8 (&clnt_authname);
@@ -1078,7 +1108,7 @@ gdm_xdmcp_send_accept (const char *hostname,
     ARRAY8 authdata;
     GdmDisplay *d;
     
-    d = gdm_xdmcp_display_alloc (hostname, displaynum);
+    d = gdm_xdmcp_display_alloc (&(clnt_sa->sin_addr), hostname, displaynum);
     
     authentype.data = (CARD8 *) 0;
     authentype.length = (CARD16) 0;
@@ -1117,12 +1147,13 @@ gdm_xdmcp_send_accept (const char *hostname,
 
 
 static void
-gdm_xdmcp_send_decline (struct sockaddr_in *clnt_sa)
+gdm_xdmcp_send_decline (struct sockaddr_in *clnt_sa, const char *reason)
 {
     XdmcpHeader header;
     ARRAY8 authentype;
     ARRAY8 authendata;
     ARRAY8 status;
+    GdmForwardQuery *fq;
     
     gdm_debug ("gdm_xdmcp_send_decline: Sending DECLINE to %s", 
 	       inet_ntoa (clnt_sa->sin_addr));
@@ -1133,7 +1164,7 @@ gdm_xdmcp_send_decline (struct sockaddr_in *clnt_sa)
     authendata.data = (CARD8 *) 0;
     authendata.length = (CARD16) 0;
     
-    status.data = "Session refused";
+    status.data = (char *)reason;
     status.length = strlen (status.data);
     
     header.version = XDM_PROTOCOL_VERSION;
@@ -1149,6 +1180,14 @@ gdm_xdmcp_send_decline (struct sockaddr_in *clnt_sa)
     
     XdmcpFlush (xdmcpfd, &buf, (XdmcpNetaddr)clnt_sa,
 		(int)sizeof (struct sockaddr_in));
+
+    /* Send MANAGED_FORWARD to indicate that the connection 
+     * reached some sort of resolution */
+    fq = gdm_forward_query_lookup (clnt_sa);
+    if (fq != NULL) {
+	    gdm_xdmcp_send_managed_forward (fq->from_sa, clnt_sa);
+	    gdm_forward_query_dispose (fq);
+    }
 }
 
 
@@ -1225,7 +1264,7 @@ gdm_xdmcp_handle_manage (struct sockaddr_in *clnt_sa, gint len)
 	 * send MANAGED_FORWARD */
 	fq = gdm_forward_query_lookup (clnt_sa);
 	if (fq != NULL) {
-		gdm_xdmcp_send_managed_forward (fq->from_sa);
+		gdm_xdmcp_send_managed_forward (fq->from_sa, clnt_sa);
 		gdm_forward_query_dispose (fq);
 	}
 
@@ -1269,24 +1308,33 @@ gdm_xdmcp_handle_manage (struct sockaddr_in *clnt_sa, gint len)
 static void 
 gdm_xdmcp_handle_managed_forward (struct sockaddr_in *clnt_sa, gint len)
 {
-	ARRAY8 clnt_hostname;
+	ARRAY8 clnt_address;
 	GdmIndirectDisplay *id;
 
+
 	gdm_debug ("gdm_xdmcp_handle_managed_forward: Got MANAGED_FORWARD from %s", inet_ntoa (clnt_sa->sin_addr));
 
 	/* Hostname */
-	if ( ! XdmcpReadARRAY8 (&buf, &clnt_hostname)) {
-		gdm_error (_("%s: Could not read hostname"),
+	if ( ! XdmcpReadARRAY8 (&buf, &clnt_address)) {
+		gdm_error (_("%s: Could not read address"),
 			   "gdm_xdmcp_handle_managed_forward");
 		return;
 	}
 
-	id = gdm_choose_indirect_lookup (clnt_sa);
+	if (clnt_address.length != sizeof (struct in_addr)) {
+		gdm_error (_("%s: Could not read address"),
+			   "gdm_xdmcp_handle_managed_forward");
+		XdmcpDisposeARRAY8 (&clnt_address);
+		return;
+	}
+
+	id = gdm_choose_indirect_lookup_by_chosen (&(clnt_sa->sin_addr),
+						   (struct in_addr *)clnt_address.data);
 	if (id != NULL) {
 		gdm_choose_indirect_dispose (id);
 	}
 
-	XdmcpDisposeARRAY8(&clnt_hostname);
+	XdmcpDisposeARRAY8 (&clnt_address);
 }
 
 
@@ -1317,6 +1365,8 @@ gdm_xdmcp_send_failed (struct sockaddr_in *clnt_sa, CARD32 sessid)
     
     gdm_debug ("gdm_xdmcp_send_failed: Sending FAILED to %ld", (long)sessid);
     
+    /* Don't translate, this goes over the wire to servers where we
+     * don't know the charset or language, so it must be ascii */
     status.data = "Failed to start session";
     status.length = strlen (status.data);
     
@@ -1412,7 +1462,7 @@ gdm_xdmcp_host_allow (struct sockaddr_in *clnt_sa)
 
 
 static GdmDisplay *
-gdm_xdmcp_display_alloc (const char *hostname, gint displaynum)
+gdm_xdmcp_display_alloc (struct in_addr *addr, const char *hostname, gint displaynum)
 {
     GdmDisplay *d = NULL;
     
@@ -1452,6 +1502,7 @@ gdm_xdmcp_display_alloc (const char *hostname, gint displaynum)
     d->name = g_strdup_printf ("%s:%d", hostname,
 			       displaynum);
     d->hostname = g_strdup (hostname);
+    memcpy (&d->addr, addr, sizeof (struct in_addr));
     
     /* Secure display with cookie */
     if (! gdm_auth_secure_display (d))
@@ -1572,5 +1623,42 @@ gdm_xdmcp_close (void)
 
 #endif /* HAVE_LIBXDMCP */
 
+gboolean
+gdm_xdmcp_is_loopback_addr (struct in_addr *ia)
+{
+	const char lo[] = {127,0,0,1};
+
+	if (ia->s_addr == INADDR_LOOPBACK ||
+	    memcmp (&ia->s_addr, lo, 4) == 0) {
+		return TRUE;
+	} else {
+		return FALSE;
+	}
+}
+
+gboolean
+gdm_xdmcp_is_local_addr (struct in_addr *ia)
+{
+	const char lo[] = {127,0,0,1};
+
+	if (ia->s_addr == INADDR_LOOPBACK ||
+	    memcmp (&ia->s_addr, lo, 4) == 0) {
+		return TRUE;
+	} else {
+		const GList *list = peek_local_address_list ();
+
+		while (list != NULL) {
+			struct in_addr *addr = list->data;
+
+			if (memcmp (&ia->s_addr, &addr->s_addr, 4) == 0) {
+				return TRUE;
+			}
+
+			list = list->next;
+		}
+
+		return FALSE;
+	}
+}
 
 /* EOF */
