@@ -706,6 +706,7 @@ gdm_config_parse_most (gboolean factory)
     gboolean got_standard_server = FALSE;
     gboolean got_any_servers = FALSE;
     char *standard_x_server = NULL;
+    int biggest_server, i;
 
     gtk_clist_clear (GTK_CLIST (get_widget ("sessions_clist")));
     gtk_clist_clear (GTK_CLIST (get_widget ("server_clist")));
@@ -813,6 +814,8 @@ gdm_config_parse_most (gboolean factory)
     gdm_spin_set("max_wait_time", gnome_config_get_int(GDM_KEY_MAXWAIT));
     gdm_spin_set("max_indirect_wait_time", gnome_config_get_int(GDM_KEY_MAXINDWAIT));
     gdm_spin_set("ping_interval", gnome_config_get_int(GDM_KEY_PINGINTERVAL));
+    gdm_spin_set("displays_per_host", gnome_config_get_int(GDM_KEY_DISPERHOST));
+    gdm_entry_set("xdmcp_willing_entry", gnome_config_get_string (GDM_KEY_WILLING));
     
     /* Fill the widgets in Sessions tab */
     gdm_toggle_set ("gnome_chooser_session", gnome_config_get_bool (GDM_KEY_SHOW_GNOME_CHOOSER));
@@ -928,7 +931,7 @@ gdm_config_parse_most (gboolean factory)
 	       GdmConfigSession *this_session;
 	       FILE *script_file;
 	       
-	       this_session = g_malloc0 (sizeof (GdmConfigSession));
+	       this_session = g_new0 (GdmConfigSession, 1);
 	       this_session->name = g_strdup(dent->d_name);
 	       script_file = fopen(s, "r");
 	       if (script_file == NULL) {
@@ -975,7 +978,6 @@ gdm_config_parse_most (gboolean factory)
     }
 
    if (default_session_name) {
-      int i;
       GdkColor col;
 
       for (i=0; i< GTK_CLIST(get_widget ("sessions_clist"))->rows; i++) {
@@ -1109,52 +1111,84 @@ gdm_config_parse_most (gboolean factory)
 	    g_free (text[1]);
 	    g_free (text[2]);
     }
-    
+
     /* Fill the widgets in Servers tab */
+    biggest_server = 0;
     iter=gnome_config_init_iterator("=" GDM_CONFIG_FILE "=/" GDM_KEY_SERVERS);
     iter=gnome_config_iterator_next (iter, &key, &value);
     
-    while (iter) {
-	char *a_server[3];
-	
-	if (isdigit (*key)) {
-		char *first = ve_first_word (ve_sure_string (value));
-		a_server[0] = key;
-		if (first[0] == '/') {
-			a_server[1] = value;
-			a_server[2] = "";
-			gtk_clist_append
-				(GTK_CLIST (get_widget ("server_clist")),
-				 a_server);
-			g_free (first);
-		} else {
-			int row;
-			char *rest = ve_rest (value);
-			char *name = get_server_name (first);
-			if (rest == NULL)
-				rest = g_strdup ("");
-			a_server[1] = name;
-			a_server[2] = rest;
-			row = gtk_clist_append
-				(GTK_CLIST (get_widget ("server_clist")),
-				 a_server);
-			gtk_clist_set_row_data_full
-				(GTK_CLIST (get_widget ("server_clist")),
-				 row,
-				 first,
-				 (GDestroyNotify) g_free);
-			g_free (name);
-			g_free (rest);
-		}
-		number_of_servers++;
-	} else {
-	      gnome_warning_dialog_parented(_("gdm_config_parse_most: Invalid server line in config file. Ignoring!"),
-					    GTK_WINDOW(GDMconfigurator));
-	}
-	g_free (key);
-	g_free (value);
-	iter=gnome_config_iterator_next (iter, &key, &value);
+    while (iter != NULL) {
+	    if (isdigit (*key)) {
+		    int svr = atoi (key);
+
+		    if (svr > biggest_server)
+			    biggest_server = svr;
+	    } else {
+		    gnome_warning_dialog_parented(_("gdm_config_parse_most: Invalid server line in config file. Ignoring!"),
+						  GTK_WINDOW(GDMconfigurator));
+	    }
+	    g_free (key);
+	    g_free (value);
+	    iter = gnome_config_iterator_next (iter, &key, &value);
     }
+    
+    /* Fill the widgets in Servers tab */
+    gnome_config_push_prefix ("=" GDM_CONFIG_FILE "=/" GDM_KEY_SERVERS "/");
+
+    for (i = 0; i <= biggest_server; i++) {
+	    char *a_server[3];
+	    char *first;
+
+	    key = g_strdup_printf ("%d", i);
+	    value = gnome_config_get_string (key);
+
+	    if (ve_string_empty (value)) {
+		    g_free (value);
+		    g_free (key);
+		    continue;
+	    }
+
+	    /* Make new, in-order, key */
+	    if (i != number_of_servers) {
+		    g_free (key);
+		    key = g_strdup_printf ("%d", number_of_servers);
+	    }
+
+	    first = ve_first_word (ve_sure_string (value));
+	    a_server[0] = key;
+	    if (first[0] == '/') {
+		    a_server[1] = value;
+		    a_server[2] = "";
+		    gtk_clist_append
+			    (GTK_CLIST (get_widget ("server_clist")),
+			     a_server);
+		    g_free (first);
+	    } else {
+		    int row;
+		    char *rest = ve_rest (value);
+		    char *name = get_server_name (first);
+		    if (rest == NULL)
+			    rest = g_strdup ("");
+		    a_server[1] = name;
+		    a_server[2] = rest;
+		    row = gtk_clist_append
+			    (GTK_CLIST (get_widget ("server_clist")),
+			     a_server);
+		    gtk_clist_set_row_data_full
+			    (GTK_CLIST (get_widget ("server_clist")),
+			     row,
+			     first,
+			     (GDestroyNotify) g_free);
+		    g_free (name);
+		    g_free (rest);
+	    }
+	    number_of_servers++;
+
+	    g_free (key);
+	    g_free (value);
+    }
+    gnome_config_pop_prefix ();
+
     /* FIXME: Could do something nicer with the 'exclude users' GUI sometime */
 
     g_free (standard_x_server);
@@ -1388,6 +1422,8 @@ write_config (void)
     gdm_spin_write("max_wait_time", GDM_KEY_MAXWAIT);
     gdm_spin_write("max_indirect_wait_time", GDM_KEY_MAXINDWAIT);
     gdm_spin_write("ping_interval", GDM_KEY_PINGINTERVAL);
+    gdm_spin_write("displays_per_host", GDM_KEY_DISPERHOST);
+    gdm_entry_write("xdmcp_willing_entry", GDM_KEY_WILLING);
     
     /* write out the widget contents of the Sessions tab */
     gdm_toggle_write ("gnome_chooser_session", GDM_KEY_SHOW_GNOME_CHOOSER);
@@ -1448,13 +1484,31 @@ write_config (void)
     
     /* Write out the widget contents of the Servers tab */
 
-    for (i=0; i<number_of_servers; i++)
-      {
-	char *current_server;
-	gtk_clist_get_text(GTK_CLIST(get_widget("server_clist")),
-			   i, 1, &current_server);
-	gnome_config_set_string(g_strdup_printf("%d", i), current_server);
-      }
+    for (i = 0; i < number_of_servers; i++) {
+	    char *key;
+	    char *val;
+	    char *extra_args = NULL;
+	    char *current_server =
+		    gtk_clist_get_row_data (GTK_CLIST (get_widget ("server_clist")), i);
+
+	    if (ve_string_empty (current_server))
+		    continue;
+
+	    gtk_clist_get_text (GTK_CLIST (get_widget ("server_clist")),
+				i, 2, &extra_args);
+
+	    if (ve_string_empty (extra_args))
+		    val = g_strdup (current_server);
+	    else
+		    val = g_strdup_printf ("%s %s", current_server,
+					   extra_args);
+
+
+	    key = g_strdup_printf ("%d", i);
+	    gnome_config_set_string (key, val);
+	    g_free (key);
+	    g_free (val);
+    }
 
    /* It would be nice to be able to do some paranoid sanity checking on
     * more of this stuff.
@@ -2376,6 +2430,19 @@ edit_selected_server (GtkButton *button, gpointer user_data)
 	handle_server_edit (TRUE /*edit*/);
 }
 
+static void
+redo_server_numbers (void)
+{
+	int i;
+
+	for (i = 0; i < number_of_servers; i++) {
+		char *num = g_strdup_printf("%d", i);
+		gtk_clist_set_text (GTK_CLIST (get_widget("server_clist")),
+				    i, 0, num);
+		g_free (num);
+	}
+}
+
 
 void
 delete_selected_server                 (GtkButton       *button,
@@ -2394,13 +2461,7 @@ delete_selected_server                 (GtkButton       *button,
     number_of_servers--;
     
     /* The server numbers will be out of sync now, so correct that */ 
-    /* FIXME: this is not right */
-    while (tmp < number_of_servers) {
-	      char *num = g_strdup_printf("%d", tmp);
-	      gtk_clist_set_text (GTK_CLIST (get_widget("server_clist")),
-				  tmp, 0, num);
-	      tmp++;
-    }
+    redo_server_numbers ();
 }
 
 
@@ -2428,14 +2489,12 @@ move_server_up                         (GtkButton       *button,
     gtk_clist_swap_rows(GTK_CLIST(get_widget("server_clist")),
 			selected_server_row,
 			selected_server_row - 1);
-    gtk_clist_set_text(GTK_CLIST(get_widget("server_clist")),
-		       selected_server_row, 0, g_strdup_printf("%d", selected_server_row));
-    
+
+    /* The server numbers will be out of sync now, so correct that */ 
+    redo_server_numbers ();
+
     gtk_clist_select_row(GTK_CLIST(get_widget("server_clist")), 
 			 --selected_server_row, 0);
-    gtk_clist_set_text(GTK_CLIST(get_widget("server_clist")),
-		       selected_server_row, 0, g_strdup_printf("%d", selected_server_row));
-
 }
 
 
@@ -2455,13 +2514,11 @@ move_server_down                       (GtkButton       *button,
     gtk_clist_swap_rows(GTK_CLIST(get_widget("server_clist")),
 			selected_server_row,
 			selected_server_row + 1);
-    gtk_clist_set_text(GTK_CLIST(get_widget("server_clist")),
-		       selected_server_row, 0, g_strdup_printf("%d", selected_server_row));
+    /* The server numbers will be out of sync now, so correct that */ 
+    redo_server_numbers ();
 
     gtk_clist_select_row(GTK_CLIST(get_widget("server_clist")), 
 			 ++selected_server_row, 0);
-    gtk_clist_set_text(GTK_CLIST(get_widget("server_clist")),
-		       selected_server_row, 0, g_strdup_printf("%d", selected_server_row));
 }
 
 void
@@ -2575,7 +2632,7 @@ add_session_real (gchar *new_session_name, gpointer data)
 	   int rowNum;
 
 	   /* printf ("Will add session with %s.\n", new_session_name); */
-	   a_session = g_malloc0 (sizeof (GdmConfigSession));
+	   a_session = g_new0 (GdmConfigSession, 1);
 	   a_session->name = g_strdup (new_session_name);
 
 	   label[0] = a_session->name;
