@@ -57,6 +57,8 @@ static Language languages [] = {
 	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|French"), "fr_FR", 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
+	{ N_("A-M|Galician"), "gl_ES", 0 },
+	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|German"), "de_DE", 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|Greek"), "el_GR", 0 },
@@ -83,6 +85,8 @@ static Language languages [] = {
 	/*Note translate the N-Z to the N-Z you used in the group label */
 	{ N_("N-Z|Portuguese"), "pt_PT", 0 },
 	/*Note translate the N-Z to the N-Z you used in the group label */
+	{ N_("N-Z|Brazilian Portuguese"), "pt_BR", 0 },
+	/*Note translate the N-Z to the N-Z you used in the group label */
 	{ N_("N-Z|Romanian"), "ro_RO", 0 },
 	/*Note translate the N-Z to the N-Z you used in the group label */
 	{ N_("N-Z|Russian"), "ru_RU", 0 },
